package inf112.skeleton.app;

import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import javafx.animation.AnimationTimer;
import javafx.application.Application;
import javafx.event.EventHandler;
import javafx.scene.*;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.effect.Bloom;
import javafx.scene.effect.BoxBlur;
import javafx.scene.image.Image;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.layout.*;
import javafx.scene.paint.*;
import javafx.scene.shape.ArcType;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.stage.Stage;

//push test
public class Game extends Application {
    private AnimationTimer timer;
    private Canvas canvas;
    private long nanosPerStep = 1000_000_000L / 60L;
    private long timeBudget = nanosPerStep;
    private long lastUpdateTime = 0L;

    public double height = 600;
    public double width = 900;

    private List<Wall> ListWalls = new ArrayList<>();
    private List<Waffle> ListWaffle = new ArrayList<>();
    private List<Enemy> ListEnemy = new ArrayList<>();
    public Player player;
    public Waffle waffle = new Waffle(1, 1, 1, 1);
    private Scene scene;

    Image hearts = new Image("file:src/PlayerImage/heart.png");
    Image waffleImg = new Image("file:src/PlayerImage/waffle.png");

    CollisionDetection cd = new CollisionDetection();

    public int gameState;
    public final int titleState = 0;
    public final int playState = 1;


    public static void startIt(String[] args) {
        launch(args);
    }

    public Scene getScene() {
    	return this.scene;
    }
    @Override
    public void start(Stage stage) {
        double width = this.width;
        double height = this.height;

        StackPane root = new StackPane();
        this.scene = new Scene(root, width, height);

        Image img = new Image("file:src/PlayerImage/backgroundPark.jpg");
        BackgroundImage bImg = new BackgroundImage(img,
                BackgroundRepeat.NO_REPEAT,
                BackgroundRepeat.NO_REPEAT,
                BackgroundPosition.CENTER,
                new BackgroundSize(1.0, 1.0, true, true, false, false));
        Background bGround = new Background(bImg);
        root.setBackground(bGround);
        stage.setScene(scene);

        canvas = new Canvas(width, height);
        canvas.widthProperty().bind(scene.widthProperty());
        canvas.heightProperty().bind(scene.heightProperty());

        setupStart();


        timer = new AnimationTimer() {

            @Override
            public void handle(long now) {
                // System.out.println("Elapsed: " + (now -
                // lastUpdateTime)/(double)millisPerStep);
                if (lastUpdateTime > 0) {
                    timeBudget = Math.min(timeBudget + (now - lastUpdateTime), 10 * nanosPerStep);
                }
                lastUpdateTime = now;

                while (timeBudget >= nanosPerStep) {
                    // System.out.println("Budget: " + timeBudget);
                    timeBudget = timeBudget - nanosPerStep;
                    step();
                }
                draw();
            }

        };
        root.getChildren().add(canvas);

        timer.start();
        stage.show();
    }

    public void setupStart() {
        gameState = titleState;
        KeyHandlerStart keyS = new KeyHandlerStart(this);
        scene.setOnKeyPressed(keyS::keyPressed);
    }

    public void setup() {
        ListEnemy.clear();
        ListWalls.clear();
        ListWaffle.clear();

        player = new Player(this);

        KeyHandler keyH = new KeyHandler(player);
        scene.setOnKeyPressed(keyH::keyPressed);
        scene.setOnKeyReleased(keyH::keyReleased);

        ListWalls.add(new Wall(250, 400, 150, 20, Color.DIMGRAY));
        ListWalls.add(new Wall(160, 570, 340, 120, Color.DIMGRAY));
        ListWalls.add(new Wall(820, 570, 300, 120, Color.DIMGRAY));
        ListWalls.add(new Wall(1200, 530, 300, 120, Color.DIMGRAY));

        ListWaffle.add(new Waffle(300, 350, 50, 50));
        ListWaffle.add(new Waffle(1000, 400, 50, 50));

        ListEnemy.add(new Enemy(700, 300, 50, 50, Color.RED, 200));
    }

    protected void step() {
        //System.out.println(gameState);
        if (gameState == playState) {

            player.canMoveLeft = true;
            player.canMoveRight = true;
            player.canMoveDown = true;
            player.canMoveUp = true;

            for (Wall wall : ListWalls) {
                if (cd.checkCollision(player, wall, "right")) {
                    player.canMoveRight = false;
                }
                if (cd.checkCollision(player, wall, "left")) {
                    player.canMoveLeft = false;
                }
                if (cd.checkCollision(player, wall, "down")) {
                    player.canMoveDown = false;
                    player.onGround = true;
                }
                if (cd.checkCollision(player, wall, "up")) {
                    player.canMoveUp = false;
                    player.jump = false;
                    player.frames = 0;
                }
            }

            for (Waffle waffle : new ArrayList<Waffle>(ListWaffle)) {
                if (cd.checkCollision(player, waffle, "left") || cd.checkCollision(player, waffle, "right") ||
                        cd.checkCollision(player, waffle, "up") || cd.checkCollision(player, waffle, "down")) {
                    player.waffleScore++;
                    ListWaffle.remove(waffle);
                }
            }

            for (Enemy enemy : new ArrayList<Enemy>(ListEnemy)) {
                enemy.walk();
                if (cd.checkCollision(player, enemy, "down") && !player.jump) {
                    ListEnemy.remove(enemy);
                }
                if (cd.checkCollision(player, enemy, "left") && !player.invincible) {
                    player.hearts--;
                    player.invincible = true;
                }
                if (cd.checkCollision(player, enemy, "right") && !player.invincible) {
                    player.hearts--;
                    player.invincible = true;
                }
                for (Wall wall : ListWalls) {
                    if (cd.checkCollision(enemy, wall, "down")) {
                        enemy.canMoveDown = false;
                    }
                }
            }

            if (player.invincible) {
                if (player.invincibleTimer == 0) {
                    player.invincible = false;
                    player.invincibleTimer = 100;
                }
                player.invincibleTimer--;
            }

            player.Move();

            if (player.getY() - player.getHeight() / 2.0 - 30 > this.height) {
                player.Dead();
            }

            if (player.hearts <= 0) {
                gameState = titleState;
                setupStart();
            }
        }
    }

    protected void draw() {
        GraphicsContext context = canvas.getGraphicsContext2D();
        context.clearRect(0, 0, canvas.getWidth(), canvas.getHeight());

        if (gameState == titleState) {
            Color c = Color.rgb(245, 187, 131);
            context.setFill(c);
            context.fillRect(0, 0, this.width, this.height);

            double x = this.width / 2 - 250;
            double y = this.height / 3;

            context.setFill(Color.WHITE);
            context.setFont(new Font("Comic sans MS", 60));
            context.fillText("Waffle Thursday", x, y);
            context.setFont(new Font("Comic sans MS", 30));
            context.fillText("Press enter to play", x + 100, y + 80);
            context.restore();
        } else {

            //WAFFLE DISPLAY
            context.drawImage(waffleImg, 10, 55, 50, 50);
            waffle.waffleCount(context, player);
            if (player.x < 200) {
                waffle.drawWaffleText(context, this.width, this.height);
            }

            //HEARTS
            for (int i = 1; i <= player.hearts; i++) {
                int w = 45;
                int h = 45;
                double x = 50 * i - w / 2.0;
                double y = 50 - h / 2.0;

                context.drawImage(hearts, x - w / 2.0, y - h / 2.0, w, h);
            }

            //PLAYER
            player.drawPlayer(context);


            //WALLS
            for (Wall wall : ListWalls) {
                int walls_w = wall.getWidth();
                int walls_h = wall.getHeight();

                double walls_xPos = (wall.getX() - walls_w / 2.0) - player.x + player.screenX;
                double walls_yPos = (wall.getY() - walls_h / 2.0);

                context.save();
                Paint c = wall.getColor();
                context.setFill(c);
                context.fillRect(walls_xPos, walls_yPos, walls_w, walls_h);
                context.restore();
            }

            //ENEMY
            for (Enemy enemy : ListEnemy) {
                int enemy_w = enemy.getWidth();
                int enemy_h = enemy.getHeight();

                double enemy_xPos = (enemy.getX() - enemy_w / 2.0) - player.x + player.screenX;
                double enemy_yPos = (enemy.getY() - enemy_h / 2.0);

                context.save();
                Paint c = enemy.getColor();
                context.setFill(c);
                context.fillRect(enemy_xPos, enemy_yPos, enemy_w, enemy_h);
                context.restore();
            }

            //WAFFLE
            for (Waffle waffle : ListWaffle) {
                int waffle_w = waffle.w;
                int waffle_h = waffle.h;

                double waffle_xPos = (waffle.x - waffle_w / 2.0) - player.x + player.screenX;
                double waffle_yPos = (waffle.y - waffle_h / 2.0);

                waffle.drawWafflePoints(context, waffle_xPos, waffle_yPos, waffle_w, waffle_h);
            }
        }
    }

}


